@@ -5,7 +5,7 @@
  * object level logging, and fully-qualified object names for all DML and DDL
  * statements where possible (See README.md for details).
  *
- * Copyright (c) 2014-2020, PostgreSQL Global Development Group
+ * Copyright (c) 2014-2017, PostgreSQL Global Development Group
  *------------------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -25,7 +25,6 @@
 #include "miscadmin.h"
 #include "libpq/auth.h"
 #include "nodes/nodes.h"
-#include "nodes/params.h"
 #include "tcop/utility.h"
 #include "tcop/deparse_utility.h"
 #include "utils/acl.h"
@@ -354,9 +353,7 @@ stack_push()
      */
     contextAudit = AllocSetContextCreate(CurrentMemoryContext,
                                          "pgaudit stack context",
-                                         ALLOCSET_DEFAULT_MINSIZE,
-                                         ALLOCSET_DEFAULT_INITSIZE,
-                                         ALLOCSET_DEFAULT_MAXSIZE);
+                                         ALLOCSET_DEFAULT_SIZES);
 
     /* Save the old context to switch back to at the end */
     contextOld = MemoryContextSwitchTo(contextAudit);
@@ -972,7 +969,7 @@ log_select_dml(Oid auditOid, List *rangeTabls)
     foreach(lr, rangeTabls)
     {
         Oid relOid;
-        Oid relNamespaceOid;
+        Relation rel;
         RangeTblEntry *rte = lfirst(lr);
 
         /* We only care about tables, and can ignore subqueries etc. */
@@ -995,10 +992,13 @@ log_select_dml(Oid auditOid, List *rangeTabls)
          * false) then filter out any system relations here.
          */
         relOid = rte->relid;
-        relNamespaceOid = get_rel_namespace(relOid);
+        rel = relation_open(relOid, NoLock);
 
-        if (!auditLogCatalog && IsSystemNamespace(relNamespaceOid))
+        if (!auditLogCatalog && IsSystemNamespace(RelationGetNamespace(rel)))
+        {
+            relation_close(rel, NoLock);
             continue;
+        }
 
         /*
          * Default is that this was not through a grant, to support session
@@ -1057,7 +1057,6 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         switch (rte->relkind)
         {
             case RELKIND_RELATION:
-            case RELKIND_PARTITIONED_TABLE:
                 auditEventStack->auditEvent.objectType = OBJECT_TYPE_TABLE;
                 break;
 
@@ -1096,8 +1095,10 @@ log_select_dml(Oid auditOid, List *rangeTabls)
 
         /* Get a copy of the relation name and assign it to object name */
         auditEventStack->auditEvent.objectName =
-            quote_qualified_identifier(
-                get_namespace_name(relNamespaceOid), get_rel_name(relOid));
+            quote_qualified_identifier(get_namespace_name(
+                                           RelationGetNamespace(rel)),
+                                       RelationGetRelationName(rel));
+        relation_close(rel, NoLock);
 
         /* Perform object auditing only if the audit role is valid */
         if (auditOid != InvalidOid)
@@ -1292,7 +1293,7 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
 
         /* Initialize the audit event */
         stackItem->auditEvent.commandText = queryDesc->sourceText;
-        stackItem->auditEvent.paramList = copyParamList(queryDesc->params);
+        stackItem->auditEvent.paramList = queryDesc->params;
     }
 
     /* Call the previous hook or standard function */
@@ -1361,30 +1362,11 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
         /* Process top level utility statement */
         if (context == PROCESS_UTILITY_TOPLEVEL)
         {
-            /*
-             * If the stack is not empty then the only allowed entries are open
-             * select, show, and explain cursors
-             */
             if (auditEventStack != NULL)
-            {
-                AuditEventStackItem *nextItem = auditEventStack;
-
-                do
-                {
-                    if (nextItem->auditEvent.commandTag != T_SelectStmt &&
-                        nextItem->auditEvent.commandTag != T_VariableShowStmt &&
-                        nextItem->auditEvent.commandTag != T_ExplainStmt)
-                    {
-                        elog(ERROR, "pgaudit stack is not empty");
-                    }
-
-                    nextItem = nextItem->next;
-                }
-                while (nextItem != NULL);
-            }
+                elog(ERROR, "pgaudit stack is not empty");
 
             stackItem = stack_push();
-            stackItem->auditEvent.paramList = copyParamList(params);
+            stackItem->auditEvent.paramList = params;
         }
         else
             stackItem = stack_push();
@@ -1403,19 +1385,6 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
             stackItem->auditEvent.commandTag == T_DoStmt &&
             !IsAbortedTransactionBlockState())
             log_audit_event(stackItem);
-
-        /*
-         * A close will free the open cursor which will also free the close
-         * audit entry. Immediately log the close and set stackItem to NULL so
-         * it won't be logged later.
-         */
-        if (stackItem->auditEvent.commandTag == T_ClosePortalStmt)
-        {
-            if (auditLogBitmap & LOG_MISC && !IsAbortedTransactionBlockState())
-                log_audit_event(stackItem);
-
-            stackItem = NULL;
-        }
     }
 
     /* Call the standard process utility chain. */
@@ -1509,9 +1478,7 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
     contextQuery = AllocSetContextCreate(
                             CurrentMemoryContext,
                             "pgaudit_func_ddl_command_end temporary context",
-                            ALLOCSET_DEFAULT_MINSIZE,
-                            ALLOCSET_DEFAULT_INITSIZE,
-                            ALLOCSET_DEFAULT_MAXSIZE);
+                            ALLOCSET_DEFAULT_SIZES);
     contextOld = MemoryContextSwitchTo(contextQuery);
 
     /* Get information about triggered events */
@@ -1620,9 +1587,7 @@ pgaudit_sql_drop(PG_FUNCTION_ARGS)
     contextQuery = AllocSetContextCreate(
                             CurrentMemoryContext,
                             "pgaudit_func_ddl_command_end temporary context",
-                            ALLOCSET_DEFAULT_MINSIZE,
-                            ALLOCSET_DEFAULT_INITSIZE,
-                            ALLOCSET_DEFAULT_MAXSIZE);
+                            ALLOCSET_DEFAULT_SIZES);
     contextOld = MemoryContextSwitchTo(contextQuery);
 
     /* Return objects affected by the drop statement */
@@ -1848,7 +1813,7 @@ _PG_init(void)
     if (inited)
         return;
 
-    /* Must be loaded with shared_preload_libraries */
+    /* Must be loaded with shared_preload_libaries */
     if (!process_shared_preload_libraries_in_progress)
         ereport(ERROR, (errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
                 errmsg("pgaudit must be loaded via shared_preload_libraries")));
